import PySimpleGUI as sg
import os.path
from pypdf import PdfReader, PdfWriter

def init_table(window_key, filenames: list):
    # 表格里添加文件，初始化数据
    global data
    total = len(filenames)
    for i in range(total):
        filename = filenames[i]
        total_page = len(PdfReader(filename).pages)
        data.append([1, filename, total_page, 1, total_page])
    refresh_table(window_key)

def refresh_table(window_key):
    # 刷新表格数据，自动排序号
    global data
    for i in range(len(data)):
        data[i][0] = i+1
    show_data = [[index, os.path.basename(filename), total_page, start_page, end_page] for index, filename, total_page, start_page, end_page in data]
    window_key.update(show_data)

def export_pdf(output_filename):
    # 合并/分割并导出PDF处理
    global data
    merger = PdfWriter()
    try:
        # 单个添加合并PDF文件
        for _, filename, _, start_page, end_page in data:
            merger.append(filename, pages=(start_page-1, end_page))
        # 输出最终文件
        with open(output_filename, 'wb') as fw:
            merger.write(fw)
        merger.close()
    except:
        merger.close()
        return False
    return True

def gui():
    global data
    # GUI主题
    sg.theme("Default1")
    # 表格标题行
    headings = ['序号(双击删除)', '{:^21}'.format("文件名"), '总页数', '起始页码(双击修改)', '结束页码(双击修改)']
    # GUI布局
    layout = [
        [sg.B("添加PDF文件", key='-ADD-'), sg.B("清空列表", key='-REMOVE-')],
        [sg.Table(values=data, headings=headings, max_col_width=10,
                    auto_size_columns=True,
                    display_row_numbers=False,
                    justification='c',
                    right_click_selects=True,
                    num_rows=20,
                    alternating_row_color='lightyellow',
                    key='-TABLE-',
                    selected_row_colors='red on yellow',
                    enable_events=True,
                    expand_x=True,
                    expand_y=True,
                    enable_click_events=True)],
        [sg.Text("导出文件路径"), sg.InputText(key='-PATH-', disabled=True, size=(90,)), sg.SaveAs('选择', file_types=(("PDF文件", "*.pdf"),), target='-PATH-')],
        [sg.Button("导出"), sg.Button("关闭")],
    ]

    # 创建窗口
    window = sg.Window("PDF合并/分割工具 {}".format(version), layout, finalize=True)
    # 绑定表格内双击事件
    window["-TABLE-"].bind('<Double-Button-1>' , "+-double click-")
    # 事件循环
    while True:
        event, values = window.read()
        # 用户关闭窗口事件
        if event == sg.WIN_CLOSED or event == "关闭":
            break
        # 点击添加文件按钮事件
        if event == '-ADD-':
            filenames = sg.popup_get_file("请选择PDF文件", multiple_files=True, file_types=(("PDF文件", "*.pdf"),), no_window=True)
            sg.popup_quick_message('加载中....', auto_close=True, non_blocking=True, font='Default 18')
            init_table(window['-TABLE-'], filenames)
            continue
        # 点击清空列表按钮事件
        elif event == '-REMOVE-':
            data = []
            refresh_table(window['-TABLE-'])
            continue
        # 点击导出按钮事件
        elif event == '导出':
            if len(data) == 0:
                sg.popup("未添加需要合并/分割的文件！", title='导出错误')
                continue
            output_filename = values['-PATH-']
            if output_filename == '':
                sg.popup("未选择导出路径！", title='导出错误')
                continue
            if export_pdf(output_filename):
                sg.popup("导出成功！", title='导出结果')
            else:
                sg.popup("导出失败！", title='导出结果')
            continue
        # 双击表格内数据事件
        elif event == '-TABLE-+-double click-':
            i, c = window['-TABLE-'].get_last_clicked_position()
            if i == None or i == -1 or c not in (0, 3, 4):
                continue
            if c == 0:
                del data[i]
                refresh_table(window['-TABLE-'])
                continue
            elif c == 3:
                input_num = sg.popup_get_text("请输入需要修改的起始页码[1-{}]".format(min(data[i][2],data[i][4])),title="请输入", modal=True, default_text=data[i][c])
            elif c == 4:
                input_num = sg.popup_get_text("请输入需要修改的结束页码[{}-{}]".format(data[i][3], data[i][2]),title="请输入", modal=True, default_text=data[i][c])
            if input_num == None:
                continue
            try:
                temp = data[i][c]
                input_num = int(input_num)
                data[i][c] = input_num
                assert (1 <= input_num <= data[i][2])
                assert (data[i][3] <= data[i][4])
            except:
                data[i][c] = temp
                sg.popup("页码有误！", title="修改失败")
                continue
            refresh_table(window['-TABLE-'])
    # 关闭窗口
    window.close()

if __name__ == "__main__":
    version = 'v1.0'
    data = []
    gui()
